<?php

namespace SplitTestForElementor\Admin\Classes\Controllers;

use SplitTestForElementor\Classes\Misc\LicenceManager;
use SplitTestForElementor\Classes\Misc\Util;
use SplitTestForElementor\Classes\Repo\PostRepo;
use SplitTestForElementor\Classes\Repo\PostTestRepo;
use SplitTestForElementor\Classes\Repo\TestManager;

class SplitTestController {

	private static $licenceManager;

	// Index; Create; Store; Show; Edit; Update; Delete

	/**
	 * SplitTestController constructor.
	 */
	public function __construct() {
		if (self::$licenceManager == null) {
			self::$licenceManager = new LicenceManager();
		}
	}

	public function run() {
		switch (isset($_GET['action']) ? $_GET['action'] : "index") {
			case "index"    :  $this->index(); break;
			case "create"   :  $this->create(); break;
			case "show"     :  $this->show(); break;
			case "edit"     :  $this->edit(); break;
			default         :  $this->index(); break;
		}
	}

	public function index() {
		$testManager = new TestManager();
		$tests = $testManager->getAllTests();
		include(__DIR__."/../../views/test/index.view.php");
	}

	public function create() {
		$postRepo = new PostRepo();
		$posts = $postRepo->getAllPosts();
		include(__DIR__."/../../views/test/create.view.php");
	}

	public function store() {

		// LOW@kberlau Input sanitation
		// LOW@kberlau use nonce

		if (!isset($_POST['test-id'])) {
			// LOW@kberlau Log Error
			wp_redirect(admin_url('admin.php?page=splittest-for-elementor&message=error_store_data_missing'));
			return;
		}

		// TODO@kberlau: Duplicate Logic
		if ($_POST['test-conversion-type'] == "page" && ($_POST['test-conversion-page'] == "" || $_POST['test-conversion-page'] == null || $_POST['test-conversion-page'] == "null")) {
			// LOW@kberlau Log Error
			wp_redirect(admin_url('admin.php?page=splittest-for-elementor&message=error_conversion_page_missing'));
			return;
		}

		// TODO@kberlau: Duplicate Logic
		if ($_POST['test-type'] == "pages") {
			if (strpos($_POST['test-uri'], '/') !== false) {
				// TODO@kberlau: Redirect to filled form
				wp_redirect(admin_url('admin.php?page=splittest-for-elementor&scope=test&action=create&message=error_test_page_invalid_chars'));
				return;
			}
			if(Util::urlExists(get_home_url()."/".$_POST['test-uri']."/")) {
				// TODO@kberlau: Redirect to filled form
				wp_redirect(admin_url('admin.php?page=splittest-for-elementor&scope=test&action=create&message=error_test_page_taken'));
				return;
			}
		}

		$testManager = new TestManager();

		// TODO@kberlau: Input validation
		if ($_POST['test-type'] == "pages") {
			// TODO@kberlau: Create / update post for preventing conflicts (custom post type maybe)
			$id = $testManager->createTest([
				'name' => $_POST['test-name'],
				'testType' => $_POST['test-type'],
				'testUri' => $_POST['test-uri'],
				'conversionType' => $_POST['test-conversion-type'],
				'conversionPageId' => $_POST['test-conversion-page']
			]);
		} else {
			$id = $testManager->createTest([
				'name' => $_POST['test-name'],
				'testType' => $_POST['test-type'],
				'conversionType' => $_POST['test-conversion-type'],
				'conversionPageId' => $_POST['test-conversion-page']
			]);
		}

		foreach ($_POST['test-variation'] as $variation) {
			$variation['postId'] = $variation['post-id'];
			$testManager->createTestVariation($id, $variation);
		}

		wp_redirect(admin_url('admin.php?page=splittest-for-elementor&scope=test&action=edit&id='.$id.'&message=store_success'));
	}

	public function show() {

	}

	public function edit() {

		$id = $_GET['id'];
		if (!filter_var($id, FILTER_VALIDATE_INT)) {
			return "Wrong Test Id";
		}

		$testManager = new TestManager();
		$test = $testManager->getTest((int) $id);

		$postRepo = new PostRepo();
		$posts = $postRepo->getAllPosts();

		$testPostRepo = new PostTestRepo();
		$postsForTest = $testPostRepo->getPostsForTest($test->id);

		include(__DIR__."/../../views/test/edit.view.php");

	}

	public function update() {

		// LOW@kberlau Input sanitation
		// LOW@kberlau use nonce

		if (!isset($_POST['test-id'])) {
			// LOW@kberlau Log Error
			wp_redirect(admin_url('admin.php?page=splittest-for-elementor&message=error_update_data_missing'));
			return;
		}

		// TODO@kberlau: Duplicate Logic
		if ($_POST['test-conversion-type'] == "page" && ($_POST['test-conversion-page'] == "" || $_POST['test-conversion-page'] == null || $_POST['test-conversion-page'] == "null")) {
			// LOW@kberlau Log Error
			wp_redirect(admin_url('admin.php?page=splittest-for-elementor&message=error_conversion_page_missing'));
			return;
		}

		if ($_POST['test-type'] == "pages") {
			if (strpos($_POST['test-uri'], '/') !== false) {
				wp_redirect(admin_url('admin.php?page=splittest-for-elementor&scope=test&action=edit&id='.$_POST['test-id'].'&message=error_test_page_invalid_chars'));
				return;
			}
			if(Util::urlExists(get_home_url()."/".$_POST['test-uri']."/")) {
				wp_redirect(admin_url('admin.php?page=splittest-for-elementor&scope=test&action=edit&id='.$_POST['test-id'].'&message=error_test_page_taken'));
				return;
			}
		}

		$testManager = new TestManager();
		// TODO@kberlau: Input validation

		if ($_POST['test-type'] == "pages") {
			// TODO@kberlau: Create / update post for preventing conflicts
			$testManager->updateTest((int) $_POST['test-id'], array(
				'name' => $_POST['test-name'],
				'testType' => $_POST['test-type'],
				'testUri' => $_POST['test-uri'],
				'conversionType' => $_POST['test-conversion-type'],
				'conversionPageId' => $_POST['test-conversion-page'] == "null" ? null : (int) $_POST['test-conversion-page']
			));
		} else {
			$testManager->updateTest((int) $_POST['test-id'], array(
				'name' => $_POST['test-name'],
				'testType' => $_POST['test-type'],
				'conversionType' => $_POST['test-conversion-type'],
				'conversionPageId' => $_POST['test-conversion-page'] == "null" ? null : (int) $_POST['test-conversion-page']
			));
		}

		foreach ($_POST['test-variation'] as $variation) {
			$variation['postId'] = (int) $variation['post-id'];
			if (Util::nullOrEmpty($variation['id'])) {
				$testManager->createTestVariation((int) $_POST['test-id'], $variation);
			} else {
				$testManager->updateTestVariation((int) $variation['id'], $variation);
			}
		}

		if (isset($_POST['test-delete-variation'])) {
			foreach ($_POST['test-delete-variation'] as $id) {
				$testManager->softDeleteTestVariation((int) $id);
			}
		}

		wp_redirect(admin_url('admin.php?page=splittest-for-elementor&scope=test&action=edit&id='.$_POST['test-id'].'&message=save_success'));
	}

	public function delete() {

		if (!isset($_GET['id'])) {
			// LOW@kberlau Log Error
			wp_redirect(admin_url('admin.php?page=splittest-for-elementor&message=error_delete'));
		}

		$testManager = new TestManager();
		$testManager->deleteTest((int) $_GET['id']);

		wp_redirect(admin_url('admin.php?page=splittest-for-elementor&scope=test&action=index&message=delete_success'));

	}

}