<?php

namespace SplitTestForElementor\Classes\Events;

use SplitTestForElementor\Classes\Misc\SettingsManager;
use SplitTestForElementor\Classes\Services\ConversionTracker;
use SplitTestForElementor\Classes\Misc\Constants;
use SplitTestForElementor\Classes\Misc\Util;
use SplitTestForElementor\Classes\Repo\PostTestManager;
use SplitTestForElementor\Classes\Repo\PostTestRepo;
use SplitTestForElementor\Classes\Repo\TestManager;

class SendHeadersEvent {

	private static $postTestManager;
	/**
	 * @var PostTestRepo
	 */
	private static $postTestRepo;
	private static $testManager;
	private static $conversionTrack;
	private static $settingsManager;

	public function __construct() {
		if (self::$postTestManager == null) {
			self::$postTestManager = new PostTestManager();
			self::$testManager = new TestManager();
			self::$conversionTrack = new ConversionTracker();
			self::$postTestRepo = new PostTestRepo();
			self::$settingsManager = new SettingsManager();
		}
	}

	public function fire() {

		$postId = url_to_postid($_SERVER['REQUEST_URI']);
		$currentLink = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? "https" : "http") . "://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]";
		$currentLink = explode("?", $currentLink)[0];
		$currentLink = trim($currentLink, "/");
		$isHomepage = site_url() == $currentLink;
		if ($isHomepage && $postId == 0) {
			$postId = (int) get_option('page_on_front');
		}

		global $clientId;
		global $rocketSplitTestClientId;
		$splitTestClientIdCookieName = Constants::$SPLIT_TEST_CLIENT_ID_COOKIE;
		if (!isset($_COOKIE[$splitTestClientIdCookieName])) {
			$clientId = substr(md5(php_uname()), 0, 9)."-".Util::generateV4UUID();
			Util::setCookie($splitTestClientIdCookieName, $clientId);
		} else {
			$clientId = $_COOKIE[$splitTestClientIdCookieName];
		}
		$rocketSplitTestClientId = $clientId;

		// TODO@kberlau: Move this to class or something
		if (empty($postId) || $postId == null || $postId == 0) {
			$this->progressTestsForRedirect( $clientId );
		}
		if (self::$settingsManager->getRawValue(SettingsManager::CACHE_BUSTER_ACTIVE)) {
			$this->prepareJsTestsForPage( $postId, $clientId );
		} else {
			$this->progressConversions( $postId, $clientId );
			$this->progressTestsForPage( $postId, $clientId );
		}
	}

	private function progressConversions($postId, $clientId) {
		$tests = self::$testManager->getTestsByConversionPagePostId($postId);
		foreach ($tests as $test) {
			$cookieName = "elementor_split_test_".$test->id."_variation";
			if(!isset($_COOKIE[$cookieName])) {
				continue;
			}

			$variationId = (int) $_COOKIE[$cookieName];
			// TODO@kberlau: Is this necessary
			foreach ($test->variations as $variation) {
				if ($variationId == $variation->id) {
					self::$conversionTrack->trackConversion($test->id, $variationId, $clientId);
				}
			}
		}
	}

	private function progressTestsForRedirect($clientId) {

		$path = parse_url($_SERVER['REQUEST_URI'], PHP_URL_PATH);
		$pathParts = explode('/', trim($path, '/'));
		$uri = end($pathParts);

		$urlQueryParams = parse_url($_SERVER['REQUEST_URI'], PHP_URL_QUERY);

		$tests = self::$testManager->getRedirectTestsByUri($uri);
		if (sizeof($tests) == 0) {
			return;
		}
		$test = $tests[0];

		$variations = $this->progressTests($tests, $clientId);

		$targetVariation = $variations[$test->id];

		if (self::$settingsManager->getRawValue(SettingsManager::CACHE_BUSTER_ACTIVE)) {
			header('Cache-Control: no-store, private, no-cache, must-revalidate');     // HTTP/1.1
			header('Cache-Control: pre-check=0, post-check=0, max-age=0, max-stale=0', false);  // HTTP/1.1
			header('Pragma: public');
			header('Expires: Sat, 26 Jul 1997 05:00:00 GMT');                  // Date in the past
			header('Expires: 0', false);
			header('Last-Modified: '.gmdate('D, d M Y H:i:s') . ' GMT');
			header('Pragma: no-cache');
			header('Vary: *');
			header("Connection: close");
		}

		// TODO@kberlau: Add no cache
		foreach ($test->variations as $variation) {
			if ($variation->id == $targetVariation->id) {
				if ($urlQueryParams == "") {
					wp_redirect(get_permalink($variation->post_id), 302);
				} else {
					wp_redirect(get_permalink($variation->post_id).'?'.$urlQueryParams, 302);
				}
				exit;
			}
		}
	}

	private function progressTestsForPage($postId, $clientId) {
		$testIds = self::$postTestRepo->getTestIdsForPost($postId);
		if (sizeof($testIds) == 0) {
			return;
		}
		$tests = self::$testManager->getTests($testIds);
		$this->progressTests($tests, $clientId);
	}

	/**
	 * @param $tests
	 * @param $clientId
	 *
	 * @return array|void
	 * @internal param $postId
	 */
	private function progressTests($tests, $clientId) {

		global $targetVariations;
		$targetVariations = [];

		foreach ($tests as $test) {
			$cookieName = "elementor_split_test_" . $test->id . "_variation";
			$targetVariation = null;

			$splitTestId = null;
			if (isset($_COOKIE[$cookieName])) {
				$splitTestId = $_COOKIE[$cookieName];
			} 
			if (isset($_GET['stid'])) {
				$splitTestId = $_GET['stid'];			
			}
			
			if ($splitTestId != null)  {
				if (filter_var($splitTestId, FILTER_VALIDATE_INT)) {
					foreach ($test->variations as $variation) {
						if ((int) $splitTestId == $variation->id) {
							$targetVariation = $variation;
							break;
						}
					}
				}
			}

			if ($targetVariation == null) {
				$rnd     = rand(1, 100);
				$counter = 0;

				$variations = $this->normalizePercentages($test->variations);

				foreach ($variations as $variation) {
					if ($rnd > $counter && $rnd <= $counter + (double) $variation->normalizedPercentage) {
						$targetVariation = $variation;
						break;
					} else {
						$counter += (double) $variation->normalizedPercentage;
					}
				}
				
			}

			if ($targetVariation != null) {
				Util::setCookie($cookieName, $targetVariation->id);
			} else {
				// LOW@kberlau ERROR!
			}

			$targetVariations[$test->id] = $targetVariation;
			self::$conversionTrack->trackView($test->id, $targetVariation->id, $clientId);
		}

		return $targetVariations;
	}

	private function prepareJsTestsForPage($postId, $clientId) {
		global $rocketSplitTestRunningTests;
		$rocketSplitTestRunningTests = [];
		$testIds = self::$postTestRepo->getTestIdsForPost($postId);
		if (sizeof($testIds) == 0) {
			return;
		}
		$tests = self::$testManager->getTests($testIds);
		foreach ($tests as $test) {
			// TODO@kberlau: Skip Page Tests
			$variations = [];
			foreach ($this->normalizePercentages($test->variations) as $variation) {
				$variations[] = [
					'id' => (int) $variation->id,
					'percentage' => $variation->normalizedPercentage,
				];
			}
			$rocketSplitTestRunningTests[] = [
				'id' => (int) $test->id,
				'variations' => $variations
			];
		}
	}

	private function normalizePercentages($variations) {
		$fullPercentageCount = 0;
		foreach ($variations as $variation) {
			$fullPercentageCount += (int) $variation->percentage;
		}

		foreach ($variations as $variation) {
			$variation->normalizedPercentage = $variation->percentage * 100 / $fullPercentageCount;
		}

		return $variations;
	}

}