<?php

namespace SplitTestForElementor\Classes\Events;

use SplitTestForElementor\Classes\Misc\SettingsManager;
use SplitTestForElementor\Classes\Repo\TestManager;

class WpHeaderEvent {

	private static $testManager;
	private static $settingsManager;

	public function __construct() {
		if (self::$testManager == null) {
			self::$testManager = new TestManager();
			self::$settingsManager = new SettingsManager();
		}
	}

	public function fire() {
		// TODO@kberlau: May not use global
		global $rocketSplitTestRunningTests;

		if (!self::$settingsManager->getRawValue(SettingsManager::CACHE_BUSTER_ACTIVE)) {
			return;
		}

		//TODO@kberlau: check if current page has tests

		$postId = url_to_postid($_SERVER['REQUEST_URI']);
		if ($postId == 0) {
			$postId = (int) get_option('page_on_front');
		}
		$conversionPageTests = self::$testManager->getTestsByConversionPagePostId($postId);
		$conversionTestIds = [];
		foreach ($conversionPageTests as $test) {
			$conversionTestIds[] = $test->id;
		}

		?>
			<script type="text/javascript">
				window.rocketSplitTest = { 'config': { 'page': { 'base': { 'protocol': 'http<?php echo(is_ssl() ? "s" : ""); ?>://', 'host': '<?php echo(parse_url(home_url('/'), PHP_URL_HOST)); ?>', 'path': '<?php echo(parse_url(home_url('/'), PHP_URL_PATH)); ?>' } } } };
				window.rocketSplitTest.cookie = { };
				window.rocketSplitTest.conversion = { };
				window.rocketSplitTest.tests = <?php echo(json_encode($rocketSplitTestRunningTests)); ?>;
				window.rocketSplitTest.conversion.testIds = <?php echo(json_encode($conversionTestIds)); ?>;
				window.rocketSplitTest.cookie.create = function (name, value, days) {
					var date = new Date();
					date.setTime(date.getTime() + (days * 24 * 60 * 60 * 1000));
					document.cookie = name + "=" + value + "; expires=" + date.toGMTString() + "; path=" + window.rocketSplitTest.config.page.base.path;
				};
				window.rocketSplitTest.cookie.read = function (name) {
					var parts = ("; " + document.cookie).split("; " + name + "=");
					return (parts.length === 2) ? parts.pop().split(";").shift() : null;
				};
		<?php if (sizeof($rocketSplitTestRunningTests) > 0) { ?>
				for (var i = 0; i < window.rocketSplitTest.tests.length; i++) {
					var test = window.rocketSplitTest.tests[i];
					var cookieName = "elementor_split_test_" + test.id + "_variation";
					var cookieValue = window.rocketSplitTest.cookie.read(cookieName);
					if (cookieValue != null) {
						for (var j = 0; j < test.variations.length; j++) {
							if (test.variations[j].id === parseInt(cookieValue)) {
								window.rocketSplitTest.tests[i].variations[j].active = true;
								break;
							}
						}
						continue;
					}
					var random = Math.floor(Math.random() * 100) + 1;
					var counter = 0;
					for (var k = 0; k < test.variations.length; k++) {
						var variation = test.variations[k];
						if (random > counter && random <= counter + variation.percentage) {
							<?php // TODO@kberlau: Test changing options ?>
							window.rocketSplitTest.tests[i].variations[k].active = true;
							window.rocketSplitTest.cookie.create(cookieName, variation.id, 365);
						} else {
							window.rocketSplitTest.tests[i].variations[k].active = false;
						}
						counter += variation.percentage;
					}
				}

				window.rocketSplitTest.isActiveVariation = function(testId, variationId) {
					for (var i = 0; i < window.rocketSplitTest.tests.length; i++) {
						var test = window.rocketSplitTest.tests[i];
						if (test.id === testId) {
							for (var k = 0; k < test.variations.length; k++) {
								var variation = test.variations[k];
								if (variation.id === variationId){
									return variation.active;
								}
							}
						}
					}
				};
		<?php } ?>
			</script>
		<?php

	}

}