<?php

namespace SplitTestForElementor\Classes\Services;

use SplitTestForElementor\Classes\Misc\SettingsManager;
use SplitTestForElementor\Classes\Services\ConversionTracker;
use SplitTestForElementor\Classes\Misc\Constants;
use SplitTestForElementor\Classes\Misc\Util;
use SplitTestForElementor\Classes\Repo\PostTestManager;
use SplitTestForElementor\Classes\Repo\PostTestRepo;
use SplitTestForElementor\Classes\Repo\TestManager;

class CustomUrlService {

	private static $postTestManager;
	/**
	 * @var PostTestRepo
	 */
	private static $postTestRepo;
	private static $testManager;
	private static $conversionTrack;
	private static $settingsManager;

	public function __construct() {
		if (self::$postTestManager == null) {
			self::$postTestManager = new PostTestManager();
			self::$testManager = new TestManager();
			self::$conversionTrack = new ConversionTracker();
			self::$postTestRepo = new PostTestRepo();
			self::$settingsManager = new SettingsManager();
		}
	}

	public function registerHooks() {

        add_action( 'init', function() {
            global $wp_rewrite;

            add_rewrite_rule(
                'split-test-for-elementor/v1/tests/([0-9]*?)/track-conversion/?$',
                'index.php?test_id=$matches[1]&rocket-split-test-action=track-conversion',
                'top'
            );

            // $wp_rewrite->flush_rules();
        } );

        add_filter( 'query_vars', function( $query_vars ){
            $query_vars[] = 'test_id';
            $query_vars[] = 'rocket-split-test-action';
            return $query_vars;
        } );
        
        add_action( 'template_redirect', function(){
            if (get_query_var('rocket-split-test-action') != "") {
                if (get_query_var('rocket-split-test-action') == "track-conversion") {
                    $testId = intval( get_query_var( 'test_id' ) );
                    $this->trackConversion($testId);
                    header("Access-Control-Allow-Origin: *");
                }
                
                header('Content-Type: image/gif');
                echo base64_decode('R0lGODlhAQABAJAAAP8AAAAAACH5BAUQAAAALAAAAAABAAEAAAICBAEAOw==');
                die();
            }
        } );

    }

    public function trackConversion($testId) {
        global $rocketSplitTestClientId;

        $test = self::$testManager->getTest($testId);
        if ($test == null) {
            return;
        }
        $cookieName = "elementor_split_test_".$test->id."_variation";
        if(!isset($_COOKIE[$cookieName])) {
            return;
        }

        $variationId = (int) $_COOKIE[$cookieName];
        foreach ($test->variations as $variation) {
            if ($variationId == $variation->id) {
                self::$conversionTrack->trackConversion($test->id, $variationId, $rocketSplitTestClientId);
            }
        }
    }
    
}